# Author: Umberto Garfagnini | University of Surrey | August 2018
# Copyright (c) 2018 University of Surrey. All rights reserved.
#-------------------------------------------------------------------------------
# This file contains the code used to generate MIE beliefs in the
# paper "Communication and Behavior in Organizations: An Experiment".   
#-------------------------------------------------------------------------------
# List of functions included in this file:
# 	(1) addMIEbeliefs : Generate MIE beliefs 
#-------------------------------------------------------------------------------

#--------------Generate MIE beliefs---------------------------------------------
# The derivation  and notation follows the theoretical steps contained in 
# the proof of Proposition 1 of Alonso, Dessein and Matouschek (AER, 2008)

#---Define and initialize some key parameters
s1 <-1          # endpoint of state space, [-s1, s1]
N <- 231        # number of partition elements of a communication equilibrium

gammaL <- 1/4    # parameter corresponding to LOW importance of coordination
gammaH <- 3/4    # parameter corresponding to HIGH importance of coordination 

#---Bias under centralization (equation 11 in ADM)
# ADM call delta the importance of coordination, which in our gamma notation
# gives delta = gamma / (1 - gamma). So, recalling that lambda = 1 in our case,
bCL <- gammaL
bCH <- gammaH

#---Bias under decentralization (equation 12 in ADM)
bDL <- 1/gammaL
bDH <- 1/gammaH

#--------------Solution of difference equations----------------------
# Next, we need to solve for the difference equations whose solutions
# provide the endpoints of the equilibrium partition elements

#---Auxiliary parameters (see pag. 171 in ADM)
# Centralization
xCL <- (1+2*bCL)+sqrt(-1+(1+2*bCL)^2)
yCL <- (1+2*bCL)-sqrt(-1+(1+2*bCL)^2)

xCH <- (1+2*bCH)+sqrt(-1+(1+2*bCH)^2)
yCH <- (1+2*bCH)-sqrt(-1+(1+2*bCH)^2)

# Decentralization
xDL <- (1+2*bDL)+sqrt(-1+(1+2*bDL)^2)
yDL <- (1+2*bDL)-sqrt(-1+(1+2*bDL)^2)

xDH <- (1+2*bDH)+sqrt(-1+(1+2*bDH)^2)
yDH <- (1+2*bDH)-sqrt(-1+(1+2*bDH)^2)


# Define a sequence of indeces from 0 to N (extremes included)
j <- seq(0, N, by=1)

# Preallocate memory for the partition endpoints of each N-element
# communication equilibrium
aCL <- rep(0,length(j)) 
aCH <- rep(0,length(j))
aDL <- rep(0,length(j))
aDH <- rep(0,length(j))

# Compute the equilibria: This uses the explicit solutions provided
# in equations A6 (for centralization) and A12 (for decentralization)
for(i in 1:length(j)){
  aCL[i]=s1*((1+yCL^N)*xCL^j[i]-(1+xCL^N)*yCL^j[i])/(xCL^N-yCL^N)
  aCH[i]=s1*((1+yCH^N)*xCH^j[i]-(1+xCH^N)*yCH^j[i])/(xCH^N-yCH^N)
  aDL[i]=s1*((1+yDL^N)*xDL^j[i]-(1+xDL^N)*yDL^j[i])/(xDL^N-yDL^N)
  aDH[i]=s1*((1+yDH^N)*xDH^j[i]-(1+xDH^N)*yDH^j[i])/(xDH^N-yDH^N)
}

# Finally, the partition equilibria computed above are used to 
# generate the MIE beliefs in the dataset which are held by the 
# receiver of the message 
# The MIE belief equals the midpoint of the partition element which 
# contains the true state of the world

addMIEbeliefs <- function(dataset){
  MIEbelief <- rep(0, nrow(dataset))
  
  for(i in 1:nrow(dataset)){
    if(dataset$vertical[i] == 0 && dataset$gamma[i] == 1){
      for(k in 1:(length(aDL)-1)){
        if(dataset$rand[i] >= aDL[k] && dataset$rand[i] <= aDL[k+1]){
          MIEbelief[i] <- (aDL[k+1]+aDL[k])/2}
      }###End of the second for loop
    }###End of IF DL
    else if(dataset$vertical[i] == 0 && dataset$gamma[i] == 3){
      for(k in 1:(length(aDH)-1)){
        if(dataset$rand[i] >= aDH[k] && dataset$rand[i] <= aDH[k+1]){
          MIEbelief[i] <- (aDH[k+1]+aDH[k])/2}
      }
      
    }
    else if(dataset$vertical[i] == 1 
            && dataset$gamma[i] == 1
            && dataset$type[i] != 0
            && dataset$type[i] != 3){
      for(k in 1:(length(aCL)-1)){
        if(dataset$rand[i] >= aCL[k] && dataset$rand[i] <= aCL[k+1]){
          MIEbelief[i] <- (aCL[k+1]+aCL[k])/2}
      }
    }
    else if(dataset$vertical[i] == 1 
            && dataset$gamma[i] == 3
            && dataset$type[i] != 0
            && dataset$type[i] != 3){
      for(k in 1:(length(aCH)-1)){
        if(dataset$rand[i] >= aCH[k] && dataset$rand[i] <= aCH[k+1]){
          MIEbelief[i] <- (aCH[k+1]+aCH[k])/2}
      } 
    }
  }###End of first for loop          
  dataset <- cbind(dataset, MIEbelief)
} ###End of function


# Load the data set
PATH_TO_FILE <- XXX # SPECIFY PATH TO DATA FILE ON YOUR COMPUTER
df <- read.csv(PATH_TO_FILE, header = TRUE, stringsAsFactors = FALSE) 

dataMIE <- addMIEbeliefs(df)