# Author: Umberto Garfagnini | University of Surrey | August 2018
# Copyright (c) 2018 University of Surrey. All rights reserved.
#-------------------------------------------------------------------------------
# This file contains the code for the simulations that are reported
# the paper "Communication and Behavior in Organizations: An Experiment".   
#-------------------------------------------------------------------------------
# Warning: Several of the simulations in this file are not optimized 
# for efficiency
#-------------------------------------------------------------------------------
# Required Libraries: plm for panel data and quantreg for QR.
#install.packages(c("plot3D", "compiler"))
library(plot3D)
library(compiler)

#--------------------------------------------------------------------------------
#----Simulation for risk-seeking preferences under CENTRALIZATION----------------
#--------------------------------------------------------------------------------

#--------------------------Centralized-Low-------------------------------------
nu1 <- seq(-0.6,0.6, by = 0.05)     # posterior belief about state theta1
nu2 <- seq(-0.6,0.6, by = 0.05)     # posterior belief about state theta2
epsilon <- seq(0.1,0.4, by = 0.02)  # parameter measuring uncertainty 
                                    # in posterior beliefs
gamma <- 1/4                        # importance of coordination
alpha <- seq(0.05,1, by = 0.05)     # risk-seeking parameter
p <- 0.5                            # probability of high posterior belief

# expected posterior belief about states theta1 and theta2
E_nu1   <- matrix(0, nrow=length(nu1), ncol=length(epsilon)) 
E_nu2   <- matrix(0, nrow=length(nu2), ncol=length(epsilon))  

# optimal decisions under risk-neutrality
optimalRN1 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
optimalRN2 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 

# optimal decisions under risk-seeking behavior
optimalRS1 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
optimalRS2 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 

# absolute coordination loss under risk neutrality
CoordLossRN <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
# average absolute coordination loss under risk neutrality
AvCoordLossRN <- matrix(0, nrow=length(alpha), ncol=length(epsilon))

# absolute coordination loss under risk seeking
CoordLossRS <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
# average absolute coordination loss under risk seeking
AvCoordLossRS <- matrix(0, nrow=length(alpha), ncol=length(epsilon)) 

# difference between CoordLossRA and CoordLossRN
CoordLoss <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
AvCoordLoss <- matrix(0, nrow=length(alpha), ncol=length(epsilon)) 


for(i in 1:length(alpha)){
  for(j in 1:length(epsilon)){
    for(k in 1:length(nu1)){ 
      for(h in 1:length(nu2)){
        funMax <- function(x){
          x1 <- x[1]
          x2 <- x[2]
          (-p^2*((1-gamma)*(x1-nu1[k]-epsilon[j])^2+(1-gamma)*(x2-nu2[h]-epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i]
          -(1-p)^2*((1-gamma)*(x1-nu1[k]+epsilon[j])^2+(1-gamma)*(x2-nu2[h]+epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i]
          -p*(1-p)*((1-gamma)*(x1-nu1[k]+epsilon[j])^2+(1-gamma)*(x2-nu2[h]-epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i]
          -p*(1-p)*((1-gamma)*(x1-nu1[k]-epsilon[j])^2+(1-gamma)*(x2-nu2[h]+epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i])
        }
        
        optimalRS <- optim(par=c(0,0), funMax,control=list(fnscale=-1), method="Nelder-Mead")$par
        optimalRS1[k,h] <- optimalRS[1]
        optimalRS2[k,h] <- optimalRS[2]
        E_nu1[k,j] <- p*(nu1[k]+epsilon[j])+(1-p)*(nu1[k]-epsilon[j])
        E_nu2[h,j] <- p*(nu2[h]+epsilon[j])+(1-p)*(nu2[h]-epsilon[j])
        optimalRN1[k,h] <- E_nu1[k]*(1+gamma)/(1+3*gamma)+E_nu2[h]*(2*gamma)/(1+3*gamma)
        optimalRN2[k,h] <- E_nu2[h]*(1+gamma)/(1+3*gamma)+E_nu1[k]*(2*gamma)/(1+3*gamma)
        CoordLoss[k,h] <- abs(optimalRS[1]-optimalRS[2])-abs(optimalRN1[k,h]-optimalRN2[k,h])
        CoordLossRS[k,h] <- abs(optimalRS1[k,h]-optimalRS2[k,h])
        CoordLossRN[k,h] <- abs(optimalRN1[k,h]-optimalRN2[k,h])
      }
    }
    AvCoordLoss[i,j] <- mean(CoordLoss)
    AvCoordLossRS[i,j] <- mean(CoordLossRS)
    AvCoordLossRN[i,j] <- mean(CoordLossRN)
  }  
}

# Average over epsilon and alpha
mean(AvCoordLoss)
mean(AvCoordLossRS)

# create 3D graph of average difference in absolute coordination losses
persp3D(alpha,epsilon,AvCoordLoss, theta=-140 , phi =10, axes = TRUE,
        main = "Average(|d1RS-d2R|-|des1RN-d2RN|), N=4, gamma=1/4", 
        xlab= "alpha", ylab= "epsilon", zlab="",
        ticktype="detailed", nticks = 3, expand= 0.85,
        col = ramp.col(c("white", "gray")), border = "black")
dev.copy(png, file='plotRS-CentralizedGammaLow.png')
dev.off()

#-------------------------------Centralized-High------------------------------
nu1 <- seq(-0.6,0.6, by = 0.05)     # posterior belief about state theta1
nu2 <- seq(-0.6,0.6, by = 0.05)     # posterior belief about state theta2
epsilon <- seq(0.1,0.4, by = 0.02)  # parameter measuring uncertainty in 
                                    # posterior beliefs
gamma <- 3/4                        # importance of coordination
alpha <- seq(0.05,1, by = 0.05)     # risk-seeking parameter
p <- 0.5                            # probability of high posterior belief

# expected posterior belief about states theta1 and theta2
E_nu1   <- matrix(0, nrow=length(nu1), ncol=length(epsilon)) 
E_nu2   <- matrix(0, nrow=length(nu2), ncol=length(epsilon))  

# optimal decisions under risk-neutrality
optimalRN1 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
optimalRN2 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 

# optimal decisions under risk-seeking behavior
optimalRS1 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
optimalRS2 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 

# absolute coordination loss under risk neutrality
CoordLossRN <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
# average absolute coordination loss under risk neutrality
AvCoordLossRN <- matrix(0, nrow=length(alpha), ncol=length(epsilon))

# absolute coordination loss under risk seeking
CoordLossRS <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
# average absolute coordination loss under risk seeking
AvCoordLossRS <- matrix(0, nrow=length(alpha), ncol=length(epsilon)) 

# difference between CoordLossRA and CoordLossRN
CoordLoss <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
AvCoordLoss <- matrix(0, nrow=length(alpha), ncol=length(epsilon)) 

 
for(i in 1:length(alpha)){
  for(j in 1:length(epsilon)){
    for(k in 1:length(nu1)){ 
      for(h in 1:length(nu2)){
        funMax <- function(x){
          x1 <- x[1]
          x2 <- x[2]
          (-p^2*((1-gamma)*(x1-nu1[k]-epsilon[j])^2+(1-gamma)*(x2-nu2[h]-epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i]
          -(1-p)^2*((1-gamma)*(x1-nu1[k]+epsilon[j])^2+(1-gamma)*(x2-nu2[h]+epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i]
          -p*(1-p)*((1-gamma)*(x1-nu1[k]+epsilon[j])^2+(1-gamma)*(x2-nu2[h]-epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i]
          -p*(1-p)*((1-gamma)*(x1-nu1[k]-epsilon[j])^2+(1-gamma)*(x2-nu2[h]+epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i])
        }
        
        optimalRS <- optim(par=c(0,0), funMax,control=list(fnscale=-1), method="Nelder-Mead")$par
        optimalRS1[k,h] <- optimalRS[1]
        optimalRS2[k,h] <- optimalRS[2]
        E_nu1[k,j] <- p*(nu1[k]+epsilon[j])+(1-p)*(nu1[k]-epsilon[j])
        E_nu2[h,j] <- p*(nu2[h]+epsilon[j])+(1-p)*(nu2[h]-epsilon[j])
        optimalRN1[k,h] <- E_nu1[k]*(1+gamma)/(1+3*gamma)+E_nu2[h]*(2*gamma)/(1+3*gamma)
        optimalRN2[k,h] <- E_nu2[h]*(1+gamma)/(1+3*gamma)+E_nu1[k]*(2*gamma)/(1+3*gamma)
        CoordLoss[k,h] <- abs(optimalRS[1]-optimalRS[2])-abs(optimalRN1[k,h]-optimalRN2[k,h])
        CoordLossRS[k,h] <- abs(optimalRS1[k,h]-optimalRS2[k,h])
        CoordLossRN[k,h] <- abs(optimalRN1[k,h]-optimalRN2[k,h])
      }
    }
    AvCoordLoss[i,j] <- mean(CoordLoss)
    AvCoordLossRS[i,j] <- mean(CoordLossRA)
    AvCoordLossRN[i,j] <- mean(CoordLossRN)
  }  
}

# Average over epsilon and alpha
mean(AvCoordLoss)
mean(AvCoordLossRS)

# create 3D graph of average difference in absolute coordination losses
persp3D(alpha,epsilon,AvCoordLoss, theta=-140 , phi =10, axes = TRUE,
        main = "Average(|d1RA-d2RA|-|des1RN-d2RN|), N=4, gamma=3/4", 
        xlab= "alpha", ylab= "epsilon", zlab="",
        ticktype="detailed", nticks = 3, expand= 0.85,
        col = ramp.col(c("white", "gray")), border = "black")
dev.copy(png, file='plotRS-CentralizedGammaHigh.png')
dev.off()




#-------------------------------------------------------------------------------
#----Simulation for risk-averse preferences under CENTRALIZATION----------------
#-------------------------------------------------------------------------------


#-------------------------------Centralized-Low-------------------------------
nu1 <- seq(-0.6,0.6, by = 0.05)     # posterior belief about state theta1
nu2 <- seq(-0.6,0.6, by = 0.05)     # posterior belief about state theta2
epsilon <- seq(0.1,0.4, by = 0.02)  # parameter measuring uncertainty in 
                                    # posterior beliefs
gamma <- 1/4                        # importance of coordination
alpha <- seq(1,5, by = 0.05)        # risk-aversion parameter
p <- 0.5                            # probability of high posterior belief

# expected posterior belief about states theta1 and theta2
E_nu1   <- matrix(0, nrow=length(nu1), ncol=length(epsilon)) 
E_nu2   <- matrix(0, nrow=length(nu2), ncol=length(epsilon))  

# optimal decisions under risk-neutrality
optimalRN1 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
optimalRN2 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 

# optimal decisions under risk-seeking behavior
optimalRA1 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
optimalRA2 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 

# absolute coordination loss under risk neutrality
CoordLossRN <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
# average absolute coordination loss under risk neutrality
AvCoordLossRN <- matrix(0, nrow=length(alpha), ncol=length(epsilon))

# absolute coordination loss under risk seeking
CoordLossRA <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
# average absolute coordination loss under risk seeking
AvCoordLossRA <- matrix(0, nrow=length(alpha), ncol=length(epsilon)) 

# difference between CoordLossRA and CoordLossRN
CoordLoss <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
AvCoordLoss <- matrix(0, nrow=length(alpha), ncol=length(epsilon)) 


for(i in 1:length(alpha)){
  for(j in 1:length(epsilon)){
    for(k in 1:length(nu1)){ 
      for(h in 1:length(nu2)){
        funMax <- function(x){
          x1 <- x[1]
          x2 <- x[2]
          (-p^2*((1-gamma)*(x1-nu1[k]-epsilon[j])^2+(1-gamma)*(x2-nu2[h]-epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i]
            -(1-p)^2*((1-gamma)*(x1-nu1[k]+epsilon[j])^2+(1-gamma)*(x2-nu2[h]+epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i]
            -p*(1-p)*((1-gamma)*(x1-nu1[k]+epsilon[j])^2+(1-gamma)*(x2-nu2[h]-epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i]
            -p*(1-p)*((1-gamma)*(x1-nu1[k]-epsilon[j])^2+(1-gamma)*(x2-nu2[h]+epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i])
        }
        
        optimalRA <- optim(par=c(0,0), funMax,control=list(fnscale=-1), method="Nelder-Mead")$par
        optimalRA1[k,h] <- optimalRA[1]
        optimalRA2[k,h] <- optimalRA[2]
        E_nu1[k,j] <- p*(nu1[k]+epsilon[j])+(1-p)*(nu1[k]-epsilon[j])
        E_nu2[h,j] <- p*(nu2[h]+epsilon[j])+(1-p)*(nu2[h]-epsilon[j])
        optimalRN1[k,h] <- E_nu1[k]*(1+gamma)/(1+3*gamma)+E_nu2[h]*(2*gamma)/(1+3*gamma)
        optimalRN2[k,h] <- E_nu2[h]*(1+gamma)/(1+3*gamma)+E_nu1[k]*(2*gamma)/(1+3*gamma)
        CoordLoss[k,h] <- abs(optimalRA[1]-optimalRA[2])-abs(optimalRN1[k,h]-optimalRN2[k,h])
        CoordLossRA[k,h] <- abs(optimalRA1[k,h]-optimalRA2[k,h])
        CoordLossRN[k,h] <- abs(optimalRN1[k,h]-optimalRN2[k,h])
      }
    }
    AvCoordLoss[i,j] <- mean(CoordLoss)
    AvCoordLossRA[i,j] <- mean(CoordLossRA)
    AvCoordLossRN[i,j] <- mean(CoordLossRN)
  }  
}

# Average over epsilon and alpha
mean(AvCoordLoss)
mean(AvCoordLossRA)

# create 3D graph of average difference in absolute coordination losses
persp3D(alpha,epsilon,AvCoordLoss, theta=-58 , phi =10, axes = TRUE,
        main = "Average(|d1_RA-d2_RA|-|des1_RN-d2_RN|), gamma=1/4", xlab= "alpha", ylab= "epsilon", zlab="",
        ticktype="detailed", nticks = 3, expand= 0.85,
        col = ramp.col(c("white", "gray")), border = "black")
dev.copy(png, file='plotRA-CentralizedGammaLow.png')
dev.off()


#-----------------------------Centralized-High--------------------------------
nu1 <- seq(-0.6,0.6, by = 0.05)     # posterior belief about state theta1
nu2 <- seq(-0.6,0.6, by = 0.05)     # posterior belief about state theta2
epsilon <- seq(0.1,0.4, by = 0.02)  # parameter measuring uncertainty in 
                                    # posterior beliefs
gamma <- 3/4                        # importance of coordination
alpha <- seq(1,5, by = 0.05)        # risk-aversion parameter
p <- 0.5                            # probability of high posterior belief

# expected posterior belief about states theta1 and theta2
E_nu1   <- matrix(0, nrow=length(nu1), ncol=length(epsilon)) 
E_nu2   <- matrix(0, nrow=length(nu2), ncol=length(epsilon))  

# optimal decisions under risk-neutrality
optimalRN1 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
optimalRN2 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 

# optimal decisions under risk-seeking behavior
optimalRA1 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
optimalRA2 <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 

# absolute coordination loss under risk neutrality
CoordLossRN <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
# average absolute coordination loss under risk neutrality
AvCoordLossRN <- matrix(0, nrow=length(alpha), ncol=length(epsilon))

# absolute coordination loss under risk seeking
CoordLossRA <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
# average absolute coordination loss under risk seeking
AvCoordLossRA <- matrix(0, nrow=length(alpha), ncol=length(epsilon)) 

# difference between CoordLossRA and CoordLossRN
CoordLoss <- matrix(0, nrow=length(nu1), ncol=length(nu2)) 
AvCoordLoss <- matrix(0, nrow=length(alpha), ncol=length(epsilon)) 


for(i in 1:length(alpha)){
  for(j in 1:length(epsilon)){
    for(k in 1:length(nu1)){ 
      for(h in 1:length(nu2)){
        funMax <- function(x){
          x1 <- x[1]
          x2 <- x[2]
          (-p^2*((1-gamma)*(x1-nu1[k]-epsilon[j])^2+(1-gamma)*(x2-nu2[h]-epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i]
            -(1-p)^2*((1-gamma)*(x1-nu1[k]+epsilon[j])^2+(1-gamma)*(x2-nu2[h]+epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i]
            -p*(1-p)*((1-gamma)*(x1-nu1[k]+epsilon[j])^2+(1-gamma)*(x2-nu2[h]-epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i]
            -p*(1-p)*((1-gamma)*(x1-nu1[k]-epsilon[j])^2+(1-gamma)*(x2-nu2[h]+epsilon[j])^2+2*gamma*(x1-x2)^2)^alpha[i])
        }
        
        optimalRA <- optim(par=c(0,0), funMax,control=list(fnscale=-1), method="Nelder-Mead")$par
        optimalRA1[k,h] <- optimalRA[1]
        optimalRA2[k,h] <- optimalRA[2]
        E_nu1[k,j] <- p*(nu1[k]+epsilon[j])+(1-p)*(nu1[k]-epsilon[j])
        E_nu2[h,j] <- p*(nu2[h]+epsilon[j])+(1-p)*(nu2[h]-epsilon[j])
        optimalRN1[k,h] <- E_nu1[k]*(1+gamma)/(1+3*gamma)+E_nu2[h]*(2*gamma)/(1+3*gamma)
        optimalRN2[k,h] <- E_nu2[h]*(1+gamma)/(1+3*gamma)+E_nu1[k]*(2*gamma)/(1+3*gamma)
        CoordLoss[k,h] <- abs(optimalRA[1]-optimalRA[2])-abs(optimalRN1[k,h]-optimalRN2[k,h])
        CoordLossRA[k,h] <- abs(optimalRA1[k,h]-optimalRA2[k,h])
        CoordLossRN[k,h] <- abs(optimalRN1[k,h]-optimalRN2[k,h])
      }
    }
    AvCoordLoss[i,j] <- mean(CoordLoss)
    AvCoordLossRA[i,j] <- mean(CoordLossRA)
    AvCoordLossRN[i,j] <- mean(CoordLossRN)
  }  
}

# Average over epsilon and alpha
mean(AvCoordLoss)
mean(AvCoordLossRA)

# create 3D graph of average difference in absolute coordination losses
persp3D(alpha,epsilon,AvCoordLoss, theta=-58 , phi =10, axes = TRUE,
        main = "Average(|d1RA-d2RA|-|des1RN-d2RN|), gamma=3/4", 
        xlab= "alpha", ylab= "epsilon", zlab="",
        ticktype="detailed", nticks = 3, expand= 0.85,
        col = ramp.col(c("white", "gray")), border = "black")
dev.copy(png, file='plotRA-CentralizedGammaHigh.png')
dev.off()




#-------------------------------------------------------------------------------
#----Simulation for risk-seeking and risk-averse preferences under DECENTRALIZATION--------------
#-------------------------------------------------------------------------------

#------------------------------Decentralized-Low--------------------------------
gamma <- 1/4                            # importance of coordination
theta1 <- seq(-1,1, by = 0.01)          # state in division manager 1
epsilon <- seq(0,0.4, by =0.02)         # measure of uncertainty in 
                                        # posterior beliefs

d2 <- seq(-0.6,0.6, by = 0.01)          # decision of division manager 2
p <- 1/2                                # probability of larger decision
alpha <- c(0.2,0.4,0.6,0.8,1,2,3,4,5)   # risk-avesion/seeking parameter

# create a grid for state theta1 and decision of division manager 2
gridd <- expand.grid(theta1, d2)

# optimal d1 decision under risk-aversion/seeking
optimalRA <- matrix(0, nrow = nrow(gridd), ncol = length(epsilon))
# optimal d1 decision under risk neutrality
optimalRN <- matrix(0, nrow = nrow(gridd), ncol = length(epsilon))

# absolute adaptation loss under risk-aversion/seeking
AdaptLossR <- matrix(0, nrow = nrow(gridd), ncol = length(epsilon))

# absolute adaptation loss under risk-neutrality
AdaptLossRN <- matrix(0, nrow = nrow(gridd), ncol = length(epsilon))

# average distance between AdaptLossR and AdaptLossRN
AvAdaptLoss <- rep(0,length(alpha))

for(k in 1:length(alpha)){
  for(j in 1:length(epsilon)){
    for(i in 1:nrow(gridd)){
      funMax <- function(x){
        (-p*((1-gamma)*(x-gridd[i,1])^2+gamma*(x-gridd[i,2]-epsilon[j])^2)^alpha[k]
        -(1-p)*((1-gamma)*(x-gridd[i,1])^2+gamma*(x-gridd[i,2]+epsilon[j])^2)^alpha[k])
      }
      #funMax <- function(x) -exp(-gamma*(-(x-theta-epsilon[i])^2-2*x^2))-exp(-gamma*(-(x-theta+epsilon[i])^2-2*x^2))
      #funMax <- function(x) log(W-(x-theta-epsilon[i])^2-2*x^2)+log(W-(x-theta+epsilon[i])^2-2*x^2)
      #optimalRA[i] <- uniroot(fun, c(-10, 10))$root
      optimalRA[i,j] <- optimize(funMax,c(-1, 1), maximum = TRUE)$maximum
      optimalRN[i,j] <- (1-gamma)*gridd[i,1]+gamma*(gridd[i,2]+(2*p-1)*epsilon[j])
      AdaptLossR[i,j] <- abs(optimalRA[i,j]-gridd[i,1])
      AdaptLossRN[i,j] <- abs(optimalRN[i,j]-gridd[i,1])
    }
    AvAdaptLoss[k,j] <- mean(AdaptLossR[,j]-AdaptLossRN[,j])
  }  
}


persp3D(alpha,epsilon,AvAdaptLoss, theta=-150 , phi =10, axes = TRUE,
        main = "Average(|d1_R-theta1|-|d1_RN-theta1|), with gamma = 1/4", 
        xlab= "alpha", ylab= "epsilon", zlab="",
        ticktype="detailed", nticks = 3, expand= 0.85,
        col = ramp.col(c("white", "gray")), border = "black")
dev.copy(png, file='plotRA-DecentralizedGammaLow.png')
dev.off()


#---------------------------Decentralized-High----------------------------------
gamma <- 3/4                            # importance of coordination
theta1 <- seq(-1,1, by = 0.01)          # state in division manager 1
epsilon <- seq(0,0.4, by =0.02)         # measure of uncertainty about 
                                        # posterior beliefs

d2 <- seq(-0.6,0.6, by = 0.01)          # decision of division manager 2
p <- 1/2                                # probability of larger decision
alpha <- c(0.2,0.4,0.6,0.8,1,2,3,4,5)   # risk-avesion/seeking parameter

# create a grid for state theta1 and decision of division manager 2
gridd <- expand.grid(theta1, d2)

# optimal d1 decision under risk-aversion/seeking
optimalRA <- matrix(0, nrow = nrow(gridd), ncol = length(epsilon))
# optimal d1 decision under risk neutrality
optimalRN <- matrix(0, nrow = nrow(gridd), ncol = length(epsilon))

# absolute adaptation loss under risk-aversion/seeking
AdaptLossR <- matrix(0, nrow = nrow(gridd), ncol = length(epsilon))

# absolute adaptation loss under risk-neutrality
AdaptLossRN <- matrix(0, nrow = nrow(gridd), ncol = length(epsilon))

# average distance between AdaptLossR and AdaptLossRN
AvAdaptLoss <- rep(0,length(alpha))

for(k in 1:length(alpha)){
  for(j in 1:length(epsilon)){
    for(i in 1:nrow(gridd)){
      funMax <- function(x){
        (-p*((1-gamma)*(x-gridd[i,1])^2+gamma*(x-gridd[i,2]-epsilon[j])^2)^alpha[k]
        -(1-p)*((1-gamma)*(x-gridd[i,1])^2+gamma*(x-gridd[i,2]+epsilon[j])^2)^alpha[k])
      }
      #funMax <- function(x) -exp(-gamma*(-(x-theta-epsilon[i])^2-2*x^2))-exp(-gamma*(-(x-theta+epsilon[i])^2-2*x^2))
      #funMax <- function(x) log(W-(x-theta-epsilon[i])^2-2*x^2)+log(W-(x-theta+epsilon[i])^2-2*x^2)
      #optimalRA[i] <- uniroot(fun, c(-10, 10))$root
      optimalRA[i,j] <- optimize(funMax,c(-1, 1), maximum = TRUE)$maximum
      optimalRN[i,j] <- (1-gamma)*gridd[i,1]+gamma*(gridd[i,2]+(2*p-1)*epsilon[j])
      AdaptLossR[i,j] <- abs(optimalRA[i,j]-gridd[i,1])
      AdaptLossRN[i,j] <- abs(optimalRN[i,j]-gridd[i,1])
    }
    AvAdaptLoss[k,j] <- mean(AdaptLossR[,j]-AdaptLossRN[,j])
  }  
}


persp3D(alpha,epsilon,AvAdaptLoss, theta=-150 , phi =10, axes = TRUE,
        main = "Average(|d1_R-theta1|-|d1_RN-theta1|), with gamma = 3/4", 
        xlab= "alpha", ylab= "epsilon", zlab="",
        ticktype="detailed", nticks = 3, expand= 0.85,
        col = ramp.col(c("white", "gray")), border = "black")
dev.copy(png, file='plotRA-DecentralizedGammaHigh.png')
dev.off()


#-------------------------------------------------------------------------------
#-------Simulation with ambiguity-averse preferences under CENTRALIZATION-------
#-------------------------------------------------------------------------------

#------Centralized Low----------------------------------------------------------
d1 <- seq(-1,1, by = 0.05)    # discretize set of decisions in division 1
d2 <- seq(-1,1, by = 0.05)    # discretize set of decisions in division 2

epsilon <- 0.4                # measure of uncertainty in posterior beliefs
bar_nu <- 1 - epsilon         # upper bound on possible posterior beliefs
gamma <- 1/4                  # importance of coordination
alpha <- c(0.2,0.4,0.6,0.8,1,2,3,4,5) # risk-aversion/seeking parameter
#p1 <- c(0.1,0.9)    # (p, 1 - p)
#p2 <- c(0.9,0.1)    # (1 - p, p)
prior <- 0.5 # both belief systems are equally likely

p <- c(0.1, 0.3, 0.6)
create_mus <- function(p) {
  out <- vector("list", length(p))
  for (i in seq_along(p)) {
    p1 <- c(p[i], 1-p[i])
    p2 <- c(1-p[i], p[i])
    out[[i]] <- list(p1 = p1, p2 = p2)
  }
  out
}
P <- create_mus(p)

# create a grid of possible decision pairs (d1, d2)
d_grid <- expand.grid(d1,d2)

# store the inner minimum in the following vector
min_mu <- rep(0,nrow(d_grid))

# create a grid of possible posterior belief pairs (nu1, nu2)
nu <- expand.grid(seq(-bar_nu,bar_nu, by = 0.05),seq(-bar_nu,bar_nu, by = 0.05))

# allocate memory for expected posterior beliefs which will be used
# to generate optimal risk neutral decisions
E_nu1 <- rep(0, nrow(nu))
E_nu2 <- rep(0, nrow(nu))

# optimal risk neutral decisions
optimalRN1 <- rep(0, nrow(nu)) 
optimalRN2 <- rep(0, nrow(nu))

# optimal ambiguity averse decisions
optimalAA1 <- rep(0, nrow(nu)) 
optimalAA2 <- rep(0, nrow(nu))

# absolute coordination loss under ambiguity aversion
CoordLossAA <- rep(0, nrow(nu))
# absolute coordination loss under risk neutrality
CoordLossRN <- rep(0, nrow(nu))
# average difference between CoordLossAA and CoordLossRN
CoordLoss <- rep(0, nrow(nu))

# store average difference in absolute coordination losses in the
# following vector
AvCoordLoss <- rep(0, length(alpha))


funMM <- function(l, x, nu) {
  x1 <- x[[1]]
  x2 <- x[[2]]
  nu1 <- nu[[1]]
  nu2 <- nu[[2]]
  return(-x1*x2*((1-gamma)*(d_grid[l,1]-nu1-epsilon)^2+(1-gamma)*(d_grid[l,2]-nu2-epsilon)^2+2*gamma*(d_grid[l,1]-d_grid[l,2])^2)^alpha[t]
         -(1-x1)*(1-x2)*((1-gamma)*(d_grid[l,1]-nu1+epsilon)^2+(1-gamma)*(d_grid[l,2]-nu2+epsilon)^2+2*gamma*(d_grid[l,1]-d_grid[l,2])^2)^alpha[t]
         -x2*(1-x1)*((1-gamma)*(d_grid[l,1]-nu1+epsilon)^2+(1-gamma)*(d_grid[l,2]-nu2-epsilon)^2+2*gamma*(d_grid[l,1]-d_grid[l,2])^2)^alpha[t]
         -x1*(1-x2)*((1-gamma)*(d_grid[l,1]-nu1-epsilon)^2+(1-gamma)*(d_grid[l,2]-nu2+epsilon)^2+2*gamma*(d_grid[l,1]-d_grid[l,2])^2)^alpha[t])
}

# compile the function funMM to byte code to make it more
# efficient using cmpfun() from "compiler" library
funMM <- cmpfun(funMM)

matAvCoordLossLOW <- matrix(0, nrow = length(alpha), ncol = length(P))

count <- 1
for (mu in P) {
  print(count)
  p1 <- mu$p1
  p2 <- mu$p2
  for(t in 1:length(alpha)) {
    for(l in seq_len(nrow(nu))) {
      print(l)
      min_mu1 <- vapply(1:nrow(d_grid), funMM, numeric(1), p1, nu[l,])
      min_mu2 <- vapply(1:nrow(d_grid), funMM, numeric(1), p2, nu[l,])
      min_mu <- pmin(min_mu1, min_mu2)
      
      # given the solution of the inner optimization,
      # find which set of decisions (d1, d2) solves the outer
      # maximization problem (recall that (nu1, nu2) are still fixed)
      for (h in seq_len(nrow(d_grid))) {
        if (min_mu[h] == max(min_mu)) {
          # found the outer maximizer
          optimalAA1[l] <- d_grid[h,1]
          optimalAA2[l] <- d_grid[h,2]
          # compute corresponding absolute coordination loss
          CoordLossAA[l] <- abs(optimalAA1[l]-optimalAA2[l])
        }
      }
      
      # find index where max(min_mu) is achieved
      max_index <- which.max(min_mu)
      optimalAA1[l] <- d_grid[max_index,1]
      optimalAA2[l] <- d_grid[max_index,2]
      # compute corresponding absolute coordination loss
      CoordLossAA[l] <- abs(optimalAA1[l]-optimalAA2[l])
      
      # generate optimal risk-neutral decisions
      E_nu1[l] <- prior * (nu[l,1] + epsilon) + (1 - prior) * (nu[l,1] - epsilon) 
      E_nu2[l] <- prior * (nu[l,2] + epsilon) + (1 - prior) * (nu[l,2] - epsilon)
      optimalRN1[l] <- E_nu1[l]*(1+gamma)/(1+3*gamma)+E_nu2[l]*(2*gamma)/(1+3*gamma)
      optimalRN2[l] <- E_nu2[l]*(1+gamma)/(1+3*gamma)+E_nu1[l]*(2*gamma)/(1+3*gamma) 
      # compute absolute coordination loss under risk-neutrality
      CoordLossRN[l] <- abs(optimalRN1[l] - optimalRN2[l])
      # compute difference between absolute coordination losses
      CoordLoss[l] <- CoordLossAA[l] - CoordLossRN[l]
    }
    # average CoordLoss over all pairs (nu1, nu2)
    AvCoordLoss[t] <- mean(CoordLoss)  
  }
  matAvCoordLossLOW[, count] <- AvCoordLoss
  count <- count + 1
}


persp3D(alpha,p,matAvCoordLossLOW, theta=-60 , phi =30, axes = TRUE,
        main = "Average(|d1_AA-d2_AA|-|des1_RN-d2_RN|)", xlab= "alpha", 
        ylab= "p", zlab="",
        ticktype="detailed", nticks = 3, expand= 0.85,
        col = ramp.col(c("white", "gray")), border = "black")
dev.copy(png, file='plot-AA-Low.png')
dev.off()


#------Centralized High---------------------------------------------------------
d1 <- seq(-1,1, by = 0.05)    # discretize set of decisions in division 1
d2 <- seq(-1,1, by = 0.05)    # discretize set of decisions in division 2

epsilon <- 0.4                # measure of uncertainty in posterior beliefs
bar_nu <- 1 - epsilon         # upper bound on possible posterior beliefs
gamma <- 3/4                  # importance of coordination
alpha <- c(0.2,0.4,0.6,0.8,1,2,3,4,5) # risk-aversion/seeking parameter
#p1 <- c(0.1,0.9)    # (p, 1 - p)
#p2 <- c(0.9,0.1)    # (1 - p, p)
prior <- 0.5 # both belief systems are equally likely

p <- c(0.1, 0.3, 0.6)
create_mus <- function(p) {
  out <- vector("list", length(p))
  for (i in seq_along(p)) {
    p1 <- c(p[i], 1-p[i])
    p2 <- c(1-p[i], p[i])
    out[[i]] <- list(p1 = p1, p2 = p2)
  }
  out
}
P <- create_mus(p)

# create a grid of possible decision pairs (d1, d2)
d_grid <- expand.grid(d1,d2)

# store the inner minimum in the following vector
min_mu <- rep(0,nrow(d_grid))

# create a grid of possible posterior belief pairs (nu1, nu2)
nu <- expand.grid(seq(-bar_nu,bar_nu, by = 0.05),seq(-bar_nu,bar_nu, by = 0.05))

# allocate memory for expected posterior beliefs which will be used
# to generate optimal risk neutral decisions
E_nu1 <- rep(0, nrow(nu))
E_nu2 <- rep(0, nrow(nu))

# optimal risk neutral decisions
optimalRN1 <- rep(0, nrow(nu)) 
optimalRN2 <- rep(0, nrow(nu))

# optimal ambiguity averse decisions
optimalAA1 <- rep(0, nrow(nu)) 
optimalAA2 <- rep(0, nrow(nu))

# absolute coordination loss under ambiguity aversion
CoordLossAA <- rep(0, nrow(nu))
# absolute coordination loss under risk neutrality
CoordLossRN <- rep(0, nrow(nu))
# average difference between CoordLossAA and CoordLossRN
CoordLoss <- rep(0, nrow(nu))

# store average difference in absolute coordination losses in the
# following vector
AvCoordLoss <- rep(0, length(alpha))


funMM <- function(l, x, nu) {
  x1 <- x[[1]]
  x2 <- x[[2]]
  nu1 <- nu[[1]]
  nu2 <- nu[[2]]
  return(-x1*x2*((1-gamma)*(d_grid[l,1]-nu1-epsilon)^2+(1-gamma)*(d_grid[l,2]-nu2-epsilon)^2+2*gamma*(d_grid[l,1]-d_grid[l,2])^2)^alpha[t]
    -(1-x1)*(1-x2)*((1-gamma)*(d_grid[l,1]-nu1+epsilon)^2+(1-gamma)*(d_grid[l,2]-nu2+epsilon)^2+2*gamma*(d_grid[l,1]-d_grid[l,2])^2)^alpha[t]
    -x2*(1-x1)*((1-gamma)*(d_grid[l,1]-nu1+epsilon)^2+(1-gamma)*(d_grid[l,2]-nu2-epsilon)^2+2*gamma*(d_grid[l,1]-d_grid[l,2])^2)^alpha[t]
    -x1*(1-x2)*((1-gamma)*(d_grid[l,1]-nu1-epsilon)^2+(1-gamma)*(d_grid[l,2]-nu2+epsilon)^2+2*gamma*(d_grid[l,1]-d_grid[l,2])^2)^alpha[t])
}

# compile the function funMM to byte code to make it more
# efficient using cmpfun() from "compiler" library
funMM <- cmpfun(funMM)

matAvCoordLossHIGH <- matrix(0, nrow = length(alpha), ncol = length(P))

count <- 1
for (mu in P) {
  p1 <- mu$p1
  p2 <- mu$p2
  for(t in 1:seq_along(alpha)) {
    for(l in seq_len(nrow(nu))) {
      #print(l)
      min_mu1 <- vapply(1:nrow(d_grid), funMM, numeric(1), p1, nu[l,])
      min_mu2 <- vapply(1:nrow(d_grid), funMM, numeric(1), p2, nu[l,])
      min_mu <- pmin(min_mu1, min_mu2)
      
      # given the solution of the inner optimization,
      # find which set of decisions (d1, d2) solves the outer
      # maximization problem (recall that (nu1, nu2) are still fixed)
      for (h in seq_len(nrow(d_grid))) {
        if (min_mu[h] == max(min_mu)) {
          # found the outer maximizer
          optimalAA1[l] <- d_grid[h,1]
          optimalAA2[l] <- d_grid[h,2]
          # compute corresponding absolute coordination loss
          CoordLossAA[l] <- abs(optimalAA1[l]-optimalAA2[l])
        }
      }
      
      # find index where max(min_mu) is achieved
      max_index <- which.max(min_mu)
      optimalAA1[l] <- d_grid[max_index,1]
      optimalAA2[l] <- d_grid[max_index,2]
      # compute corresponding absolute coordination loss
      CoordLossAA[l] <- abs(optimalAA1[l]-optimalAA2[l])
      
      # generate optimal risk-neutral decisions
      E_nu1[l] <- prior * (nu[l,1] + epsilon) + (1 - prior) * (nu[l,1] - epsilon) 
      E_nu2[l] <- prior * (nu[l,2] + epsilon) + (1 - prior) * (nu[l,2] - epsilon)
      optimalRN1[l] <- E_nu1[l]*(1+gamma)/(1+3*gamma)+E_nu2[l]*(2*gamma)/(1+3*gamma)
      optimalRN2[l] <- E_nu2[l]*(1+gamma)/(1+3*gamma)+E_nu1[l]*(2*gamma)/(1+3*gamma) 
      # compute absolute coordination loss under risk-neutrality
      CoordLossRN[l] <- abs(optimalRN1[l] - optimalRN2[l])
      # compute difference between absolute coordination losses
      CoordLoss[l] <- CoordLossAA[l] - CoordLossRN[l]
    }
    # average CoordLoss over all pairs (nu1, nu2)
    AvCoordLoss[t] <- mean(CoordLoss)  
  }
  matAvCoordLossHIGH[, count] <- AvCoordLoss
  count <- count + 1
}


persp3D(alpha,p,matAvCoordLossHIGH, theta=-60 , phi =30, axes = TRUE,
        main = "Average(|d1_AA-d2_AA|-|des1_RN-d2_RN|)", xlab= "alpha", 
        ylab= "p", zlab="",
        ticktype="detailed", nticks = 3, expand= 0.85,
        col = ramp.col(c("white", "gray")), border = "black")
dev.copy(png, file='plot-AA-High.png')
dev.off()















